/* Copyright (c) 2013 Asustor Inc. All rights reserved. */
Ext.define('AS.ARC.apps.linuxCenter.core.OnOffDisplayField', {
    extend: 'Ext.form.field.Display',
    alias: ['widget.asLinuxCenterOnOffDisplay'],

    valueToRaw: function (value) {
        if (value === true) {
            this.addClass('enable-status-text');
            this.removeCls('disable-status-text');
            return _S('LINUX_CENTER', 'ACTIVATE');
        } else if (value === false) {
            this.addClass('disable-status-text');
            this.removeCls('enable-status-text');
            return _S('LINUX_CENTER', 'NOT_ACTIVATE');
        } else {
            this.addClass('disable-status-text');
            this.removeCls('enable-status-text');
            return _S('LINUX_CENTER', 'NOT_SUPPORTED');
        }
    }
});

Ext.define('AS.ARC.apps.linuxCenter.core.StatusTip', {
    extend: 'AS.ARC.statusTip',
    alias: ['widget.asLinuxCenterStatusTip'],

    showMsgAfterApply: function(imgSrcHTML, valueHTML){
		var controller = this,
            field = controller.tipField,
            value = imgSrcHTML +' '+ valueHTML;
        
		if( controller.tipField === null ){
			return false;
		};

		if( controller.idTimer !== null ){
			clearTimeout(controller.idTimer);
		}
        
        field.hide();
        field.setValue(value);
        field.el.fadeIn({useDisplay: true});
        field.show();
        
		controller.idTimer = setTimeout(function () {
		    if(typeof field.el !== 'undefined'){
                field.el.fadeOut({useDisplay: false});
				if(Ext.isIE8  || Ext.isIE7 || Ext.isIE6){
                    field.hide();
				}
			}
			controller.idTimer = null;
			}, 5000);
	},

    // 顯示固定的訊息，不會自動淡出
    showErrorMsgStatic: function (error_msg, error_code) {
        var msg = (typeof error_msg === 'undefined' ? _S('ERROR', 'UNKNOWN_ERROR') : error_msg) + 
                (typeof error_code !== 'undefined' && error_code !== '' ? ' (Ref. ' + error_code + ')' : ''),
            imgSrcHTML = '<img src="' + AS.ARC.util.getUserAppsPath() + 'linux-center/images/common-icon_16X16_alert.png" alt="" >',
		    valueHTML = '<span style="color:#ef0000; position:relative; bottom:3px">' + msg + '</span>',
            controller = this,
            field = controller.tipField,
            value = imgSrcHTML +' '+ valueHTML;

        field.setValue(value).show();
    },

    bindMethodWith: function(panel){

		var me = this;
		
		// Dynamic delegate showTip method.
		panel.showApply = function(){
			me.showApply();
		};
		panel.showSetNotChange = function(){
			me.showSetNotChange();
		};

        panel.showErrorMsgStatic = function (error_msg, error_code) {
            me.showErrorMsgStatic(error_msg, error_code);
        }

        panel.clearTip = function () {
            me.tipField.hide();
        }
	},
})

/**
 * @class AS.ARC.apps.miniDLNA.core
 * @extends Ext.util.Observable
 * <p>core</p>
 */
Ext.define('AS.ARC.apps.linuxCenter.core', {
    extend: 'Ext.util.Observable',

    apiUrl: AS.ARC.util.getUserAppsPath() + 'linux-center/lxc.cgi',
    imgUrl: AS.ARC.util.getUserAppsPath() + 'linux-center/images',

    constructor: function (config) {
        Ext.apply(this, config);
        this.callParent();
        this.init(config);
    },

    init: function () {
        var fn = this;

        this.win = this.desktop.createWindow({
            app: fn.app,
            id: fn.id,
            itemId: fn.id,
            width: 880,
            height: 550,
            resizable: true,
            border: false,
            layout: 'fit',
            items: [
                fn.getMainPanel()
            ],
            listeners: {
                afterrender: function (win) {
                    win.el.dom.querySelector('.x-window-header').classList.add('as-window-header-no-tab');

                    // 即使狀態沒有變更也要重新更新畫面
                    fn.forceUpdatePanel = true;
                    fn.loadData();
                },
                beforeclose: function () {
                    fn.stopStatusPolling();
                }
            }
        });
    },

    currentContainer: {
        index: 0,
        name: '',
        displayName: '',
        status: ''
    },

    getVNCUrl: function () {
        var vncPortNumber = (location.protocol === 'http:' ? 8071 : 8072);
        return location.protocol + '//' + location.hostname + ':' + vncPortNumber + '/asustor_linux/';
    },

    handleInstall: function (config) {
        var fn = this,
            containerName = fn.currentContainer.name;

        var getProgress = function () {
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get-progress'}),
                method: 'get',
                params: {name: containerName},
                success: function (json, opts) {
                    var downloadProgressBar = installWin.down('#download-progress'),
                        installProgressBar = installWin.down('#install-progress'),
                        installLabel = installWin.down('#install-label'),
                        progress = json.progress;
                    
                    if (isNaN(progress) || progress < 0) {
                        progress = 0;
                    } else if (progress > 1) {
                        progress = 1;
                    }

                    if (json.status === 'download') {
                        downloadProgressBar.updateProgress(progress);
                        downloadProgressBar.updateText((progress * 100).toFixed() + ' %');

                        fn.getProgressTimeoutId = setTimeout(getProgress, 3000);
                    } else if (json.status === 'install') {
                        downloadProgressBar.updateProgress(1);
                        downloadProgressBar.updateText('100 %');
                        installWin.down('#download-status').addCls('shadow');
                        installWin.down('#install-label').setValue(_S('LINUX_CENTER', 'INSTALLING'));
                        installProgressBar.updateProgress(progress);
                        installProgressBar.updateText((progress * 100).toFixed() + ' %');

                        fn.getProgressTimeoutId = setTimeout(getProgress, 3000);
                    } else if (json.status === 'finish') {
                        downloadProgressBar.updateProgress(1);
                        downloadProgressBar.updateText('100 %');
                        installWin.down('#download-status').addCls('shadow');
                        installWin.down('#install-label').setValue(_S('LINUX_CENTER', 'INSTALLING'));
                        installProgressBar.updateProgress(1);
                        installProgressBar.updateText('100 %');
                        installWin.close();

                        // 如果要由使用者手動關閉安裝視窗，則將此註解取消
                        // installWin.down('#install-close').enable();
                        
                        // 即使狀態沒有變更也要重新更新畫面
                        fn.forceUpdatePanel = true;
                        fn.loadData();
                    } else if (json.status === 'error') {
                        installWin.showErrorMsgStatic(json.error_msg, json.error_code);
                        installWin.down('#install-close').enable();
                    }
                },
                failure: function (json, opts) {
                    var messageField = installWin.down('#install-error-message'),
                        error_msg = '', 
                        error_code = '';

                    if (json.error_code === 5016) {
                        error_msg = _S('LINUX_CENTER', 'CONNECT_FAIL');
                    } else if (json.error_code === 5601) {
                        error_msg = _S('LINUX_CENTER', 'TIMEOUT');
                    } else if (json.error_code === 6202) {
                        error_msg = _S('LINUX_CENTER', 'DOWNLOAD_FAIL');
                    } else if (json.error_code === 5043) {
                        error_msg = Ext.String.format(_S('LINUX_CENTER', 'VOLUME_FULL'), '1');
                    } else {
                        error_msg = json.error_msg;
                        error_code = json.error_code;
                    }
                    installWin.showErrorMsgStatic(error_msg, error_code);
                    installWin.down('#install-close').enable();
                }
            });
        };

        var install = function () {
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: config.act}),
                method: 'get',
                params: {name: containerName, clear: config.clear},
                success: function (json, opts) {
                    getProgress();
                },
                failure: function (json, opts) {
                    var error_msg = json.error_msg + ' (Ref. ' + json.error_code + ')';

                    if (json.error_code === 5008) {
                        // already installing (not error, just show install win)
                        getProgress();
                    } else {
                        installWin.showErrorMsgStatic(json.error_msg, json.error_code);
                        installWin.down('#install-close').enable();
                    }
                }
            });
        };

        var handleCancelInstall = function() {
            var remove = (config.act === 'install' ? true : false);

            clearTimeout(fn.getProgressTimeoutId);
            installWin.down('#install-cancel').disable();

            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'stop-install', name: containerName, remove: remove}),
                method: 'get',
                success: function() {
                    installWin.close();
                },

                failure: function() {
                    if (json.error_code === 5401) {
                        installWin.showErrorMsgStatic(_S('LINUX_CENTER', 'CANCEL_INSTALL_FAILED'), json.error_code);
                    } else {
                        var error_msg = json.error_msg + ' (Ref. ' + json.error_code + ')';
                        installWin.showErrorMsgStatic(json.error_msg, json.error_code);
                    }
                    installWin.down('#install-close').enable();
                }
            });
        };

        var installWinStatusTip = Ext.create( 'AS.ARC.apps.linuxCenter.core.StatusTip');

        var installWin = Ext.create('AS.ARC.subWindow', {
            title: (config.act === 'install' ? _S('COMMON', 'INSTALL') : _S('LINUX_CENTER', 'RESTORE_SETTING')),
            id: 'linux-center-install-win',
            parentWin: fn.app.win,
            width: 600,
            height: 375,
            layout: 'fit',
            fbar: [{
                text: _S('COMMON', 'CANCEL'),
                itemId: 'install-cancel',
                handler: handleCancelInstall
            }, {
                text: _S('COMMON', 'CLOSE'),
                itemId: 'install-close',
                disabled: true,
                handler: function () {
                    installWin.close();
                }
            }],
            items: [{
                xtype: 'asPanelTop20',
                items: [{
                    xtype: 'container',
                    cls: 'download-status',
                    itemId: 'download-status',
                    items: [{
                        xtype: 'displayfield',
                        cls: 'displayField',
                        value: _S('LINUX_CENTER', 'DOWNLOADING')
                    }, {
                        xtype: 'progressbar',
                        itemId: 'download-progress',
                        cls: 'update-bar',
                        value: 0,
                        text: '0 %'
                    }]
                }, {
                    xtype: 'container',
                    cls: 'install-status',
                    items: [{
                        xtype: 'displayfield',
                        cls: 'displayField',
                        itemId: 'install-label',
                        value: _S('LINUX_CENTER', 'WAIT_FOR_INSTALL')
                    }, {
                        xtype: 'progressbar',
                        itemId: 'install-progress',
                        cls: 'update-bar',
                        value: 0,
                        text: '0 %'
                    }]
                }, {
                    xtype: 'container',
                    cls: 'install-error-message',
                    items: [installWinStatusTip.getTipField()]
                }]
            }],
            listeners: {
                afterrender: function () {
                    if (config.act === 'install' || config.act === 'reset') {
                        install();
                    } else {
                        getProgress();
                    }
                }
            }
        });
        installWinStatusTip.bindMethodWith(installWin);
        installWin.show();
    },

    handleSetting: function () {
        var fn = this,
            containerName = fn.currentContainer.name;

        var setSettings = function (containerName, settingData) {
            var params = settingData;
            params.name = containerName;

            settingsWin.close();
            fn.app.maskWin(_S('COMMON', 'APPLYING'));

            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'set-settings'}),
                method: 'get',
                params: settingData,
                win: settingsWin,
                success: function (json, opts) {
                    // 即使狀態沒有變更也要重新更新畫面
                    fn.forceUpdatePanel = true;
                    fn.loadData();
                    fn.infoPanel.showApply();
                },
                failure: function (json, opts) {
                    fn.infoPanel.showErrorMsgStatic(_S('LINUX_CENTER', 'APPLY_SETTING_FAIL'), json.error_code);
                    fn.app.unmaskWin();
                }
            });
        };

        var getNet = function () {
            settingsWin.el.mask(_S('COMMON', 'LOADING'));
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(AS.ARC.util.getApiUrl('network.cgi', 'settings'), {act: 'get', tab: 'General'}),
                method: 'get',
                win: settingsWin,
                params: {name: containerName},
                success: function (json, opts) {
                    var data = [];
                    Ext.Array.each(json.IfList, function (net) {
                        data.push([net.Name]);
                    });
                    settingsWin.down('#net').getStore().loadData(data);
                    
                    settingsWin.el.unmask();
                },
                failure: function () {
                    settingsWin.el.unmask();
                }
            });
        }

        var handleApply = function () {
            var containerStatus = fn.currentContainer.status,
                newSettings = settingsWin.down('#setting-panel').getNewSattings(),
                originalSettings = settingsWin.down('#setting-panel').originalSettings,
                settingData = {},
                doSetSettings = false;

            // 如果不支援 hdmi 時，移除 dpi, hdmi, remote 欄位
            if (fn.currentContainer.supportHdmi === false) {
                delete newSettings.dpi;
                delete newSettings.hdmi;
                delete newSettings.remote;
            }

            // 取出不同的參數放到新的物件中
            Ext.iterate(newSettings, function (key, value) {
                if (newSettings[key] !== originalSettings[key]) {
                    settingData[key] = value;
                    doSetSettings = true;
                }
            });

            if (doSetSettings) {
                if (containerStatus === 'running' && ('net' in settingData || 'share_folder' in settingData)) {
                    var confirmWin = new AS.ARC.msgWindow({
                        parentWin: settingsWin,
                        title: _S('COMMON', 'CONFIRMATION'),
                        width: 480,
                        height: 200,
                        iconType: 'question',
                        asItems: [{
                            xtype: 'displayfield',
                            value: _S('LINUX_CENTER', 'NOTE_APPLY_RESET')
                        }],
                        fbar: [{
                            xtype: 'button',
                            text: _S('COMMON', 'YES'),
                            handler: function() {
                                confirmWin.close();
                                setSettings(containerName, settingData);
                            }
                        }, {
                            xtype: 'button',
                            text: _S('COMMON', 'NO'),
                            handler: function() {
                                confirmWin.close();
                            }
                        }]				
                    }).show();
                } else {
                    setSettings(containerName, settingData);
                }
            } else {
                fn.infoPanel.showSetNotChange();
                settingsWin.close();
                fn.startStatusPolling();
            }
        };

        var buttonApply = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'OK'),
            handler: function () {
                handleApply();
            },
            scope: this
        });

        var settingsWin = Ext.create('AS.ARC.subWindow', {
            title: _S('LINUX_CENTER', 'CONFIGURE'),
            id: 'linux-center-setting-win',
            parentWin: fn.app.win,
            width: 640,
            height: 500,
            layout: 'fit',
            fbar: [
                buttonApply,
                {
                    text: _S('COMMON', 'CANCEL'),
                    handler: function () {
                        settingsWin.close();
                        fn.startStatusPolling();
                    }
                }
            ],
            items: [{
                xtype: 'asPanelTop20',
                itemId: 'setting-panel',
                items:[{
                    xtype: 'container',
                    cls: 'setting-block',
                    itemId: 'display-block',
                    items: [{
                        xtype: 'displayfield',
                        value: AS.ARC.util.fontToBold(_S('LINUX_CENTER', 'DISPLAY_SETTING'))
                    }, {
                        xtype: 'combo',
                        itemId: 'dpi',
                        fieldLabel: _S('LINUX_CENTER', 'RESOLUTION'),
                        labelWidth: 240,
                        editable: false,
                        style: AS.layout.getIndentCss(1),
                        queryMode: 'local',
                        displayField: 'display',
                        store: Ext.create('Ext.data.ArrayStore', {
                            fields: ['display'],
                            data: [
                                ['1920x1080'],
                                ['1920x1080i'],
                                ['1600x1200'],
                                ['1680x1050'],
                                ['1280x1024'],
                                ['1440x900'],
                                ['1280x960'],
                                ['1366x768'],
                                ['1152x864'],
                                ['1280x720']
                            ]
                        })
                    }]
                }, {
                    xtype: 'container',
                    cls: 'setting-block',
                    itemId: 'hdmi-block',
                    items: [{
                        xtype: 'checkbox',
                        itemId: 'hdmi',
                        boxLabel: AS.ARC.util.fontToBold(_S('LINUX_CENTER', 'ENABLE_HDMI_VIDEO_OUTPUT')),
                    }, {
                        xtype: 'displayfield',
                        cls: 'note-string indented',
                        style: AS.layout.getIndentCss(1),
                        value: _S('LINUX_CENTER', 'HDMI_NOTE')
                    }]
                }, {
                    xtype: 'container',
                    cls: 'setting-block',
                    itemId: 'remote-link-block',
                    items: [{
                        xtype: 'checkbox',
                        itemId: 'remote',
                        boxLabel: AS.ARC.util.fontToBold(_S('LINUX_CENTER', 'REMOTE_DESKTOP_FUNC'))
                    }, {
                        xtype: 'container',
                        itemId: 'remote-link',
                        cls: 'remote-link',
                        items: [{
                            xtype: 'displayfield',
                            style: AS.layout.getIndentCss(1),
                            value: _S('LINUX_CENTER', 'REMOTE_DESKTOP_NOTE')
                        }, {
                            xtype: 'displayfield',
                            itemId: 'remote-url',
                            style: AS.layout.getIndentCss(1),
                            value: Ext.String.format(_S('LINUX_CENTER', 'REMOTE_DESKTOP_USE_URL'), fn.getVNCUrl())  
                        }]
                    }]
                }, {
                    xtype: 'container',
                    cls: 'setting-block',
                    items: [{
                        xtype: 'displayfield',
                        value: AS.ARC.util.fontToBold(_S('LINUX_CENTER', 'NETWORK_SETTING'))
                    }, {
                        xtype: 'combo',
                        itemId: 'net',
                        fieldLabel: _S('LINUX_CENTER', 'NETWORK_INTERFACE'),
                        labelWidth: 240,
                        editable: false,
                        style: AS.layout.getIndentCss(1),
                        queryMode: 'local',
                        displayField: 'display',
                        store: Ext.create('Ext.data.ArrayStore', {
                            fields: ['display'],
                            data: []
                        })
                    }]
                }, {
                    xtype: 'container',
                    cls: 'setting-block',
                    items: [{
                        xtype: 'checkbox',
                        itemId: 'share-folder',
                        boxLabel: AS.ARC.util.fontToBold(_S('LINUX_CENTER', 'MOUNT_SHARE_FOLDER'))
                    }]
                }],
                listeners: {
                    afterrender: function (panel) {
                        
                        settingsWin.el.mask(_S('COMMON', 'LOADING'));
                        panel.originalSettings = fn.infoPanel.originalSettings;

                        panel.down('#dpi').setValue(panel.originalSettings.dpi);
                        panel.down('#hdmi').setValue(panel.originalSettings.hdmi);
                        panel.down('#net').setValue(panel.originalSettings.net);
                        panel.down('#remote').setValue(panel.originalSettings.remote);
                        panel.down('#share-folder').setValue(panel.originalSettings.share_folder);

                        if (fn.currentContainer.supportHdmi) {
                            if (fn.currentContainer.status === 'running' && fn.infoPanel.originalSettings.remote === true) {
                                panel.down('#remote-url').setValue(Ext.String.format(_S('LINUX_CENTER', 'REMOTE_DESKTOP_USE_URL'), fn.getVNCUrl()));
                                fn.enableFields(panel.down('#remote-link'));
                            } else {
                                panel.down('#remote-url').setValue(Ext.String.format(_S('LINUX_CENTER', 'REMOTE_DESKTOP_USE_URL'), ''));
                                fn.disableFields(panel.down('#remote-link'));
                            }
                        } else {
                            fn.disableFields(panel.down('#display-block'));
                            fn.disableFields(panel.down('#hdmi-block'));
                            fn.disableFields(panel.down('#remote-link-block'));
                        }
                        
                        getNet();
                    }
                },
                getNewSattings: function () {
                    return {
                        dpi: this.down('#dpi').getValue(),
                        hdmi: this.down('#hdmi').getValue(),
                        net: this.down('#net').getValue(),
                        remote: this.down('#remote').getValue(),
                        share_folder: this.down('#share-folder').getValue(),
                    }
                }
            }]
        });
        settingsWin.show();
        fn.stopStatusPolling();
    },

    handleInitialize: function () {
        var fn = this,
            containerName = fn.currentContainer.name;
        
        var confirmWin = new AS.ARC.msgWindow({
            parentWin: fn.app.win,
            title: _S('COMMON', 'CONFIRMATION'),
            width: 500,
            height: 260,
            iconType: 'question',
            asItems: [{
                xtype: 'displayfield',
                value: _S('LINUX_CENTER', 'NOTE_RESTORE')
            }, {
                xtype: 'checkbox',
                itemId: 'clear',
                boxLabel: _S('LINUX_CENTER', 'RETAIN_SETTING')
            }],
            fbar: [{
                xtype: 'button',
                text: _S('COMMON', 'YES'),
                handler: function() {
                    var clear = confirmWin.down('#clear').getValue() ? false : true;
                    confirmWin.close();
                    fn.handleInstall({act: 'reset', clear: clear});
                }
            }, {
                xtype: 'button',
                text: _S('COMMON', 'NO'),
                handler: function() {
                    confirmWin.close();
                    fn.startStatusPolling();
                }
            }]				
        }).show();
        fn.stopStatusPolling();
    },

	getGridSelectorStore: function () {
		var fn = this;

		return Ext.create('Ext.data.Store', {
			fields: ['name', 'status', 'display_name', 'support_hdmi'],
			autoLoad: false,
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'list-containers'}),
				reader: {
					type: 'json',
					root: 'containers'
				}
			},
			listeners: {
				load: function (store) {
                    if (store.getCount() > 0) {
                        fn.gridSelector.getSelectionModel().select(fn.currentContainer.index);
                    }
				}
			}
		});
	},

	getGridSelector: function () {
		var fn = this;

		return Ext.create('Ext.grid.Panel', {
			itemId: 'selector-grid',
			hideHeaders: true,
			border: false,
			store: this.getGridSelectorStore(),
			height: 380,
			columns: [{
				dataIndex: 'name',
				renderer: function (value, cell, record) {
					return '<div class="linux-container-item">' +
								'<div class="linux-container-icon status-' + record.get('status')  + '"></div>' +
								'<div class="linux-container-name">' + record.get('display_name') + '</div>' +
						   '</div>';
				},
				flex: 1
			}],
			listeners: {
				select: function (me, record, index) {

                    var selectedContainer =  {
                        index: index,
                        name: record.get('name'),
                        displayName: record.get('display_name'),
                        status: record.get('status'),
                        supportHdmi: record.get('support_hdmi')
                    }

                    // 比對選擇的 container 改變、或所選 container 的狀態改變 (ex: running -> shutdown) 才需更新畫面
                    if (fn.forceUpdatePanel || JSON.stringify(fn.currentContainer) !== JSON.stringify(selectedContainer)) {
                        fn.currentContainer = selectedContainer;

                        if (record.get('status') === 'uninstall') {
                            fn.showInstallPanel();
                        } else if (record.get('status') === 'shutdown') {
                            fn.buttonStop.hide();
                            fn.buttonStart.show();
                            fn.buttonReStart.disable();

                            fn.showInfoPanel();
                        } else { // status === running
                            fn.buttonStop.show();
                            fn.buttonStart.hide();
                            fn.buttonReStart.enable();

                            fn.showInfoPanel();
                        }

                        // 更新完畫面將旗標重置
                        fn.forceUpdatePanel = false;
                    }

                    if (record.get('status') === 'running') {
                        fn.startStatusPolling();
                    } else {
                        fn.stopStatusPolling();
                    }
				}
			},
		});
	},	

    getSettings: function () {
        var fn = this,
            containerName = fn.currentContainer.name,
            containerStatus = fn.currentContainer.status;

        fn.app.maskWin(_S('COMMON', 'LOADING'));

        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get-settings'}),
            method: 'get',
            params: {name: containerName},
            success: function (json, opts) {
                fn.infoPanel.originalSettings = json;
                
                if (fn.currentContainer.supportHdmi) {
                    fn.infoPanel.down('#dpi').setValue(json.dpi);
                    fn.infoPanel.down('#hdmi').setValue(json.hdmi);
                    fn.infoPanel.down('#remote').setValue(json.remote);
                } else {
                    fn.infoPanel.down('#dpi').setValue(AS.ARC.datas.defaultValue);
                    fn.infoPanel.down('#hdmi').setValue(null);
                    fn.infoPanel.down('#remote').setValue(null);
                }
                
                fn.infoPanel.down('#net').setValue(json.net);
                fn.infoPanel.down('#share-folder').setValue(json.share_folder);

                if (containerStatus === 'running' && fn.infoPanel.originalSettings.remote === true) {
                    fn.infoPanel.down('#remote-url').setValue(Ext.String.format(_S('LINUX_CENTER', 'REMOTE_DESKTOP_USE_URL'), fn.getVNCUrl()));
                    fn.enableFields(fn.infoPanel.down('#remote-link'));
                } else {
                    fn.infoPanel.down('#remote-url').setValue(Ext.String.format(_S('LINUX_CENTER', 'REMOTE_DESKTOP_USE_URL'), ''));
                    fn.disableFields(fn.infoPanel.down('#remote-link'));
                }

                fn.app.unmaskWin();
            },
            failure: function (json, opts) {
                fn.infoPanel.showErrorMsgStatic(json.error_msg, json.error_code);
                fn.app.unmaskWin();
            }
        });

        var getIp = function () {
            if (fn.currentContainer.status === 'running') {
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get-ip'}),
                    method: 'get',
                    params: {name: fn.currentContainer.name},
                    success: function (json, opts) {
                        fn.infoPanel.down('#ip').setValue(json.ip);
                    },
                    failure: function (json, opts) {
                        fn.infoPanel.down('#ip').setValue(AS.ARC.datas.defaultValue);
                        
                        // need polling until success
                        var pollingTimeoutId = setTimeout(function () { getIp(); }, 2000);
                        fn.app.win.on('close', function () { clearTimeout(pollingTimeoutId); });
                    }
                }); 
            }
        };

        if (containerStatus === 'running') {
            getIp();
        } else {
            fn.infoPanel.down('#ip').setValue(AS.ARC.datas.defaultValue);
        }        
    },

    handleStart: function () {
        var fn = this,
            containerName = fn.currentContainer.name;

        fn.app.maskWin(_S('COMMON', 'APPLYING'));
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'start-container'}),
            method: 'get',
            params: {name: containerName},
            success: function (json, opts) {
                fn.infoPanel.clearTip();

                // 即使狀態沒有變更也要重新更新畫面
                fn.forceUpdatePanel = true;
                fn.loadData();
            },
            failure: function (json, opts) {
                fn.infoPanel.showErrorMsgStatic(json.error_msg, json.error_code);

                // 即使狀態沒有變更也要重新更新畫面
                fn.forceUpdatePanel = true;
                fn.loadData();
            }
        });
    },

    // 傳入的 config.act 為 stop 時為將虛擬機關閉, 
    // 為 restart 時為將虛擬機重啟, 需要再呼叫 handleStart()
    handleStop: function (config) {
        var fn = this,
            containerName = fn.currentContainer.name,
            stopRequestId,
            forceStopTimeoutId;

        var stopContainer = function () {
            fn.app.maskWin(_S('COMMON', 'APPLYING'));
            return Ext.Ajax.request({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'stop-container'}),
                method: 'get',
                params: {name: containerName},
                timeout: 3 * 60 * 1000, // 60 秒後會跳出詢問視窗詢問是否強制關閉，所以將 timeout 調長
                success: function (json, opts) {
                    // prevent forrce stop confirm window show up
                    forceStopConfirmWin.close();
                    clearTimeout(forceStopTimeoutId);

					fn.infoPanel.clearTip();

                    if (config.act === 'stop') {

                        // 即使狀態沒有變更也要重新更新畫面
                        fn.forceUpdatePanel = true;
                        fn.loadData();
                    } else if (config.act === 'restart') {
                        fn.handleStart();
                    }
                },
                failure: function (json, opts) {
                    var error_msg = _S('LINUX_CENTER', 'PROCESSING');
					fn.infoPanel.showErrorMsgStatic(error_msg, json.error_code);

                    // 即使狀態沒有變更也要重新更新畫面
                    fn.forceUpdatePanel = true;
                    fn.loadData();
                }
            });
        };

        var forceStopContainer = function () {
            fn.app.maskWin(_S('COMMON', 'APPLYING'));
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'force-stop-container'}),
                method: 'get',
                params: {name: containerName},
                success: function (json, opts) {
                    fn.infoPanel.clearTip();
                    
                    if (config.act === 'stop') {

                        // 即使狀態沒有變更也要重新更新畫面
                        fn.forceUpdatePanel = true;
                        fn.loadData();
                    } else if (config.act === 'restart') {
                        
                        fn.handleStart();
                    }
                },
                failure: function (json, opts) {
                    var error_msg = _S('LINUX_CENTER', 'PROCESSING');
                    fn.infoPanel.showErrorMsgStatic(error_msg, json.error_code);

                    // 即使狀態沒有變更也要重新更新畫面
                    fn.forceUpdatePanel = true;
                    fn.loadData();
                }
            });
        }

        var forceStopConfirmWin = new AS.ARC.msgWindow({
            parentWin: fn.app.win,
            title: _S('COMMON', 'CONFIRMATION'),
            width: 480,
            height: 200,
            iconType: 'question',
            asItems: [{
                xtype: 'displayfield',
                value: _S('LINUX_CENTER', 'FORCE_STOP_NOTE')
            }],
            fbar: [{
                xtype: 'button',
                text: _S('COMMON', 'YES'),
                handler: function() {
                    forceStopConfirmWin.close();
                    Ext.Ajax.abort(stopRequestId);
                    forceStopContainer();
                }
            }, {
                xtype: 'button',
                text: _S('COMMON', 'NO'),
                handler: function() {
                    forceStopConfirmWin.close();
                }
            }]				
        });

        var confirmWin = new AS.ARC.msgWindow({
            parentWin: fn.app.win,
            title: _S('COMMON', 'CONFIRMATION'),
            width: 480,
            height: 200,
            iconType: 'question',
            asItems: [{
                xtype: 'displayfield',
                value: (config.act === 'stop' ? _S('LINUX_CENTER', 'DISABLE_NOTE') : _S('LINUX_CENTER', 'RE_ENABLE_NOTE'))   
            }],
            fbar: [{
                xtype: 'button',
                text: _S('COMMON', 'YES'),
                handler: function() {
                    confirmWin.close();
                    stopRequestId = stopContainer();
                    forceStopTimeoutId = setTimeout(function () {forceStopConfirmWin.show();}, 60 * 1000);
                }
            }, {
                xtype: 'button',
                text: _S('COMMON', 'NO'),
                handler: function() {
                    confirmWin.close();
                    fn.startStatusPolling();
                }
            }]				
        });
        confirmWin.show();
        fn.stopStatusPolling();
    },

    getLastInstallStatus: function () {
        var fn = this,
            containerName = fn.currentContainer.name;

        fn.app.maskWin(_S('COMMON', 'LOADING'));

        // call get-progress to get last installed status
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, {act: 'get-progress'}),
            method: 'get',
            params: {name: containerName},
            success: function (json, opts) {
                if (json.status === 'download' || json.status === 'install') {
                    fn.handleInstall({act: 'showProgress', clear: false});
                } else if (json.status === 'finish') {
                    if (fn.currentContainer.status === 'shutdown' || fn.currentContainer.status === 'running') {
                        fn.getSettings();
                    }
                } else if (json.status === 'error') {
                    if (fn.currentContainer.status === 'shutdown' || fn.currentContainer.status === 'running') {
                        fn.infoPanel.showErrorMsgStatic(json.error_msg, json.error_code);
                        fn.getSettings();
                    } else if (fn.currentContainer.status === 'uninstall') {
                        fn.installPanel.showErrorMsgStatic(json.error_msg, json.error_code);
                        fn.app.unmaskWin();
                    }                    
                }
            },
            failure: function (json, opts) {
                // if current container was never installed, this ajax will always fail,
                // need not to show error messages
                fn.app.unmaskWin();

                if (fn.currentContainer.status !== 'uninstall') {
                    fn.getSettings();
                }
            }
        });
    },

    disableFields: function (fields) {
        var fn = this;

        Ext.each(fields, function (field) {
            if (field.xtype === 'container') {
                return fn.disableFields(field.items.items);
            } else {
                field.disable();
            }
        });
        return true;
    },

    enableFields: function (fields) {
        var fn = this;

        Ext.each(fields, function (field) {
            if (field.xtype === 'container') {
                return fn.enableFields(field.items.items);
            } else {
                field.enable();
            }
        });
		return true;
    },

    showInstallPanel: function () {
		var fn = this,
            displayName = fn.currentContainer.displayName;

        fn.getLastInstallStatus();
        fn.installPanel.down('#title-container-name').update(displayName);
		fn.panel.down('#container').layout.setActiveItem(fn.installPanel);
	},

	showInfoPanel: function () {
		var fn = this;
        fn.getLastInstallStatus();
		fn.panel.down('#container').layout.setActiveItem(fn.infoPanel);
	},

    startStatusPolling: function () {
        var fn = this;

        clearTimeout(fn.statusTimeout);
        fn.statusTimeout = setTimeout(function () {
            fn.loadData();
        }, 10 * 1000);
    },

    stopStatusPolling: function () {
        var fn = this;

        clearTimeout(fn.statusTimeout);
    },

    loadData: function () {
        var fn = this;

        fn.gridSelector.store.load();
    },

    getMainPanel: function () {
        var fn = this;

		fn.gridSelector = fn.getGridSelector();

        fn.buttonInstall = Ext.create('Ext.Button', {
            text: _S('COMMON', 'INSTALL'),
            handler: function () {
                fn.handleInstall({act: 'install', clear: false});
            }
        });

		fn.buttonSetting = Ext.create('Ext.Button', {
			text: _S('LINUX_CENTER', 'CONFIGURE'),
            handler: function () {
                fn.handleSetting();
            }
		});

		fn.buttonStop = Ext.create('Ext.Button', {
			text: _S('COMMON', 'DISABLE'),
            handler: function () {
                fn.handleStop({act: 'stop'});
            }
		});

		fn.buttonStart = Ext.create('Ext.Button', {
			text: _S('COMMON', 'ENABLE'),
            handler: function () {
                fn.handleStart();
            }
		});

		fn.buttonReStart = Ext.create('Ext.Button', {
			text: _S('LINUX_CENTER', 'RESET'),
            handler: function () {
                fn.handleStop({act: 'restart'});
            }
		});

		fn.buttonInitialize = Ext.create('Ext.Button', {
			text: _S('LINUX_CENTER', 'RESTORE_SETTING'),
            handler: function () {
                fn.handleInitialize();
            }
		});

        fn.dpiStore = Ext.create('Ext.data.ArrayStore', {
            autoDestroy: true,
            fields: ['available_dpi']
        });

        fn.netStore = Ext.create('Ext.data.ArrayStore', {
            autoDestroy: true,
            fields: ['net']
        });

        var installPanelStatusTip = Ext.create( 'AS.ARC.apps.linuxCenter.core.StatusTip');

		fn.installPanel = Ext.create('Ext.panel.Panel', {
			cls: 'as-page-panel install-panel',
            border: false,
            dockedItems: [{
                xtype: 'toolbar',
                dock: 'bottom',
                ui: 'footer',
                height: 60,
                items: [
                    installPanelStatusTip.getTipField(),
                    {xtype: 'component', flex: 1},
                    fn.buttonInstall
                ]
            }],
			items: [{
                xtype: 'container',
                cls: 'linux-center-title',
                items: [{
                    xtype: 'component',
                    autoEl: 'h1',
                    cls: 'linux-center-title-main',
                    html: 'Linux',
                }, {
                    xtype: 'component',
                    autoEl: 'h2',
                    itemId: 'title-container-name',
                    cls: 'linux-center-title-container-name',
                    html: AS.ARC.datas.defaultValue
                }]
            }, {
                xtype: 'box',
                cls: 'linux-center-title-hr',
                autoEl: {tag: 'hr'}
            }, {
                xtype: 'container',
                cls: 'linux-center-desc',
                items: [{
                    xtype: 'container',
                    cls: 'linux-center-intro',
                    items: [{
                        xtype: 'displayfield',
                        cls: 'displayField',
                        value: Ext.String.format(_S('LINUX_CENTER', 'INTRO'), 'mailto:lxcdevel@lists.linuxcontainers.org')
                    }, {
                        xtype: 'displayfield',
                        cls: 'displayField',
                        value: _S('LINUX_CENTER', 'INTRO2')
                    }]
                }]
            }]
		});

        installPanelStatusTip.bindMethodWith(fn.installPanel);

        var infoPanelStatusTip = Ext.create( 'AS.ARC.apps.linuxCenter.core.StatusTip');

		fn.infoPanel = Ext.create('Ext.panel.Panel', {
            cls: 'info-panel',
            refId: 'fnTabPanel',
            border: false,
            layout: {type: 'vbox', align: 'stretch'},
            dockedItems: [{
                xtype: 'toolbar',
                height: 60,
                dock: 'bottom',
                ui: 'footer',
                items: [
                    infoPanelStatusTip.getTipField()
                ]
            }],
			items: [{
				xtype: 'asPanelToolbarTop',
				itemId: 'tbar',
				items: [
					fn.buttonSetting,
					fn.buttonStop,
					fn.buttonStart,
					fn.buttonReStart,
					fn.buttonInitialize
				]
			}, {
                xtype: 'panel',
                cls: 'as-page-panel info-panel-content',
                flex: 1,
                border: false,
                items: [{
                    xtype: 'container',
                    layout: 'column',
                    items: [{
                        xtype: 'fieldset',
                        title: _S('LINUX_CENTER', 'DISPLAY_SETTING'),
                        columnWidth: 0.5,
                        defaults: {labelWidth: 200},
                        items: [{
                            xtype: 'displayfield',
                            itemId: 'dpi',
                            fieldLabel: _S('LINUX_CENTER', 'RESOLUTION'),
                            value: AS.ARC.datas.defaultValue
                        }, {
                            xtype: 'asLinuxCenterOnOffDisplay',
                            anchor: '100%',
                            itemId: 'hdmi',
                            fieldLabel: _S('LINUX_CENTER', 'HDMI_VIDEO_OUTPUT')
                        }]
                    }, {
                        xtype: 'box',
                        width: 35,
                        height: 1
                    }, {
                        xtype: 'fieldset',
                        title: _S('LINUX_CENTER', 'REMOTE_CONNECTION'),
                        columnWidth: 0.5,
                        defaults: {labelWidth: 200},
                        items: [{
                            xtype: 'displayfield',
                            itemId: 'ip',
                            fieldLabel: _S('LINUX_CENTER', 'IP_ADDRESS'),
                            value: AS.ARC.datas.defaultValue
                        }, {
                            xtype: 'displayfield',
                            value: _S('LINUX_CENTER', 'CONNECT_SSH'),
                            cls: 'ssh-note-string'
                        }]
                    }]
                }, {
                    xtype: 'fieldset',
                    title: _S('LINUX_CENTER', 'REMOTE_DESKTOP'),
                    defaults: {labelWidth: 200},
                    items: [{
                        xtype: 'asLinuxCenterOnOffDisplay',
                        itemId: 'remote',
                        fieldLabel: _S('LINUX_CENTER', 'REMOTE_DESKTOP_FUNC'),
                    }, {
                        xtype: 'container',
                        itemId: 'remote-link',
                        cls: 'remote-link',
                        items: [{
                            xtype: 'displayfield',
                            value: _S('LINUX_CENTER', 'REMOTE_DESKTOP_NOTE')
                        }, {
                            xtype: 'displayfield',
                            itemId: 'remote-url',
                            value: Ext.String.format(_S('LINUX_CENTER', 'REMOTE_DESKTOP_USE_URL'), fn.getVNCUrl())
                        }, {
                            xtype: 'displayfield',
                            cls: 'note-string',
                            value: _S('LINUX_CENTER', 'VNC_NOTE')
                        }]
                    }]
                }, {
                    xtype: 'container',
                    layout: 'column',
                    items: [{
                        xtype: 'fieldset',
                        title: _S('LINUX_CENTER', 'NETWORK_SETTING'),
                        columnWidth: 0.5,
                        defaults: {labelWidth: 200},
                        items: [{
                            xtype: 'displayfield',
                            itemId: 'net',
                            fieldLabel: _S('LINUX_CENTER', 'NETWORK_INTERFACE'),
                        }]
                    }, {
                        xtype: 'box',
                        width: 35,
                        height: 1
                    }, {
                        xtype: 'fieldset',
                        title: _S('LINUX_CENTER', 'SHARE_FOLDER'),
                        columnWidth: 0.5,
                        defaults: {labelWidth: 200},
                        items: [{
                            xtype: 'asLinuxCenterOnOffDisplay',
                            itemId: 'share-folder',
                            fieldLabel: _S('LINUX_CENTER', 'MOUNT_SHARE_FOLDER')
                        }]
                    }]
                }, {
                    xtype: 'container',
                    cls: 'note-default-account-password',
                    items: [{
                        xtype: 'displayfield',
                        cls: 'note-string',
                        value: _S('LINUX_CENTER', 'NOTE_DEFAULT_ACCOUNT_PASSWORD')
                    }]
                }]
            }]
		});

        infoPanelStatusTip.bindMethodWith(fn.infoPanel);

        fn.panel = Ext.create('Ext.panel.Panel', {
			itemId: 'main',
            layout: 'border',
            border: false,
			items: [{
                region: 'west',
                itemId: 'selector',
                cls: 'as-selector-panel',
                border: false,
                split: true,
                maxWidth: 350,
                minWidth: 150,
                width: 250,
                layout: 'fit',
                items: [
                    fn.gridSelector
                ]
            }, {
                region: 'center',
                xtype: 'panel',
                itemId: 'container',
                cls: 'as-function-panel',
                layout: 'card',
                items: [
                    {xtype: 'panel', border: false}, // 此 panel 是用來顯示一開始進入 app 時的空白畫面
                    fn.installPanel,
                    fn.infoPanel
                ]
            }]
        });

        return fn.panel;
    }
});

/**
 * @class AS.ARC.apps.linuxCenter.main
 * @extends AS.ARC._appBase
 * <p>Main app</p>
 */
Ext.define('AS.ARC.apps.linuxCenter.main', {
	extend: 'AS.ARC._appBase',
	//user-linux-center
	appTag: 'linux-center',
	title: _S('LINUX_CENTER', 'LINUX_CENTER'),

	createWindow: function () {
		var desktop = this.core.getDesktop(),
			win = desktop.getWindow(this.id);

		if (!win) {
			var newLC = new AS.ARC.apps.linuxCenter.core({
				app: this,
				desktop: desktop,
				title: this.title,
				id: this.id
			});

			// Set win to this app
			this.win = newLC.win;

			win = newLC.win;
		}
		win.show();
	}
});